package com.chelvc.cloud.vehicle.client;

import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.NoticeDTO;
import com.chelvc.cloud.vehicle.client.param.NoticeModifyParam;
import com.chelvc.cloud.vehicle.client.param.NoticePagingParam;
import com.chelvc.framework.common.model.Pagination;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 通知配置业务接口
 *
 * @author liude
 * @data 2023/12/7
 */
@FeignClient("vehicle")
public interface NoticeClient {

    /**
     * 新增通知
     *
     * @param param 新增参数
     * @return 主键
     */
    @PostMapping("/notice/addNotice")
    Long addNotice(@RequestBody NoticeModifyParam param);

    /**
     * 修改通知
     *
     * @param id    主键
     * @param param 修改参数
     */
    @PutMapping("/notice/updateNotice")
    void updateNotice(@RequestParam("id") Long id, @RequestBody NoticeModifyParam param);

    /**
     * 查询通知
     *
     * @param param 查询参数
     * @return 通知分页信息
     */
    @GetMapping("/notice/getNoticePaging")
    Pagination<NoticeDTO> getNoticePaging(@SpringQueryMap NoticePagingParam param);

    /**
     * 查询通知列表
     *
     * @return 通知信息
     */
    @GetMapping("/notice/getNoticeList")
    List<NoticeDTO> getNoticeList(@RequestParam("type") String type);
}
