package com.chelvc.cloud.vehicle.client.dto;
import com.chelvc.framework.common.model.File;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 商家认证数据模型
 *
 * @author liude
 * @data 2023/12/10
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
public class MerchantAuthDTO implements Serializable {
    /**
     * 主键
     */
    private Long id;
    /**
     * 营业执照编号
     */
    private String businessLicenseId;
    /**
     * 营业执照证件号
     */
    private String businessLicenseNumber;
    /**
     * 营业执照有效期
     */
    private Date businessLicenseExpirationDate;
    /**
     * 身份证号
     */
    private String idCardNumber;
    /**
     * 身份证有效期
     */
    private Date idCardExpirationDate;
    /**
     * 店铺图
     */
    private String storeImage;
    /**
     * 法人姓名
     */
    private String legalRepresentativeName;
    /**
     * 法人性别
     */
    private String legalRepresentativeGender;
    /**
     * 手机号
     */
    private String mobileNumber;
    /**
     * 账号
     */
    private String account;
    /**
     * 账号注册日期
     */
    private Date accountRegistrationDate;
    /**
     * 店铺类型
     */
    private String storeType;
    /**
     *店名
     */
    private String storeName;
    /**
     * 店铺地址
     */
    private String storeAddress;
    /**
     * 经营许可证照片
     */
    private String businessLicensePhoto;
    /**
     * 经营许可证号
     */
    private String businessLicenseNumberValid;
    /**
     * 经营许可证有效期
     */
    private Date businessLicenseExpirationDateValid;
    /**
     *  审核状态
     */
    private String reviewStatus; // 审核状态
    /**
     * 审核人
     */
    private String reviewer; // 审核人
    /**
     * 推荐人
     */
    private String recommendationUser; // 推荐人
    /**
     * 审核时间
     */
    private Date reviewTime; // 审核时间
    /**
     * 申请时间
     */
    private Date applicationTime; //
    /**
     * 商家信息
     */
    private MerchantDTO merchant;
    /**
     * 申请人
     */
    private Long applicant;


    /**
     * 营业执照证件号
     */
    private String businessPhoto;
    /**
     * 身份证正面照片
     */
    private String idCardFrontPhoto;

    /**
     * 身份证反面照片
     */
    private String idCardBackPhoto;

    /**
     * 店铺内部环境图
     */
    private List<File> storeInnerImage;

    /**
     * 店铺详细地址
     */
    @Size(max = 255, message = "店铺详细地址长度不能大于255")
    private String storeAddressDetail;

    /**
     * 邮箱
     */
    @Size(max = 255, message = "邮箱长度不能大于255")
    private String email;

    /**
     * 审核意见
     */
    private String message;

}
