package com.chelvc.cloud.vehicle.server.controller;

import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.HelpCategoryDTO;
import com.chelvc.cloud.vehicle.client.param.HelpCategoryModifyParam;
import com.chelvc.cloud.vehicle.client.param.HelpCategoryPagingParam;
import com.chelvc.cloud.vehicle.server.service.HelpCategoryService;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 帮助分类Service接口
 *
 * @author liude
 * @date 2023-11-08
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class HelpCategoryController {
    private final HelpCategoryService helpCategoryService;

    /**
     * 查询帮助分类
     *
     * @param id 帮助分类主键
     * @return 帮助分类
     */
    @GetMapping("/helpCategory/selectHelpCategoryById")
    public HelpCategoryDTO selectHelpCategoryById(@RequestParam("id") Long id) {
        return helpCategoryService.selectHelpCategoryById(id);
    }

    /**
     * 查询帮助分类列表
     *
     * @param helpCategory 帮助分类
     * @return 帮助分类集合
     */
    @PostMapping("/helpCategory/selectHelpCategoryList")
    public List<HelpCategoryDTO> selectHelpCategoryList(HelpCategoryModifyParam helpCategory) {
        return helpCategoryService.selectHelpCategoryList(helpCategory);
    }

    /**
     * 新增帮助分类
     *
     * @param helpCategory 帮助分类
     * @return 结果
     */
    @PostMapping("/helpCategory/insertHelpCategory")
    public void insertHelpCategory(@RequestBody HelpCategoryModifyParam helpCategory) {
        helpCategoryService.insertHelpCategory(helpCategory);
    }

    /**
     * 修改帮助分类
     *
     * @param helpCategory 帮助分类
     * @return 结果
     */
    @PutMapping("/helpCategory/updateHelpCategory")
    public void updateHelpCategory(@RequestParam("id") Long id, @RequestBody HelpCategoryModifyParam helpCategory) {
        helpCategoryService.updateHelpCategory(id, helpCategory);
    }


    /**
     * 删除帮助分类信息
     *
     * @param id 帮助分类主键
     * @return 结果
     */
    @DeleteMapping("/helpCategory")
    public void deleteHelpCategoryById(@RequestParam("id") Long id) {
        helpCategoryService.deleteHelpCategoryById(id);
    }

    /**
     * 分页查询帮助分类列表
     *
     * @param param
     * @return
     */
    @GetMapping("/helpCategory/getHelpCategoryPaging")
    public Pagination<HelpCategoryDTO> getHelpCategoryPaging(HelpCategoryPagingParam param) {
        return helpCategoryService.getHelpCategoryPaging(param);
    }


    /**
     * 查询帮助分类列表
     *
     * @return 帮助分类集合
     */
    @GetMapping("/helpCategory/selectHelpCategories")
    public List<HelpCategoryDTO> selectHelpCategories() {
        return helpCategoryService.selectHelpCategories();
    }
}
