package com.chelvc.cloud.vehicle.server.controller;


import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.MerchantCartItemDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsCartItemDTO;
import com.chelvc.cloud.vehicle.client.param.OmsCartItemModifyParam;
import com.chelvc.cloud.vehicle.server.service.OmsCartItemService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 购物车Service接口
 *
 * @author liude
 * @date 2023-11-08
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OmsCartItemController {
    private final OmsCartItemService omsCartItemService;

    /**
     * 查询购物车中是否包含该商品，有增加数量，无添加到购物车
     */
    @PostMapping("/omsCartItem/add")
    public int add(@RequestBody OmsCartItemModifyParam cartItem) {
        return omsCartItemService.add(cartItem);
    }

    /**
     * 根据会员编号获取购物车列表
     */
    @GetMapping("/omsCartItem/listCartItem")
    public List<OmsCartItemDTO> listCartItem() {
        return omsCartItemService.listCartItem();
    }


    /**
     * 修改某个购物车商品的数量
     */
    @PostMapping("/omsCartItem/updateQuantity")
    public int updateQuantity(@RequestParam("id") Long id, @RequestParam("quantity") Integer quantity) {
        return omsCartItemService.updateQuantity(id, quantity);
    }

    /**
     * 批量删除购物车中的商品
     */
    @DeleteMapping("/omsCartItem")
    public void delete(@RequestParam("ids") List<Long> ids) {
        omsCartItemService.delete(ids);
    }

    /**
     * 修改购物车中商品的规格
     */
    @PostMapping("/omsCartItem/updateAttr")
    public int updateAttr(@RequestBody OmsCartItemModifyParam cartItem) {
        return omsCartItemService.updateAttr(cartItem);
    }

    /**
     * 清空购物车
     */
    @PostMapping("/omsCartItem/clear")
    public void clear() {
        omsCartItemService.clear();
    }

    /**
     * 根据会员编号及商家获取购物车列表
     */
    @GetMapping("/omsCartItem/listMerchantCartItem")
    public MerchantCartItemDTO listMerchantCartItem(@RequestParam("merchantId") Long merchantId) {
        return omsCartItemService.listCartItem(merchantId);
    }
}
