package com.chelvc.cloud.vehicle.server.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.chelvc.cloud.vehicle.client.dto.CategoryDTO;
import com.chelvc.cloud.vehicle.client.param.CategoryListParam;
import com.chelvc.cloud.vehicle.client.param.CategoryModifyParam;
import com.chelvc.cloud.vehicle.client.param.CategoryPagingParam;
import com.chelvc.cloud.vehicle.server.entity.Category;
import com.chelvc.framework.common.model.Pagination;

import java.util.List;

/**
 * 分类业务操作接口
 *
 * @author Woody
 * @date 2023/5/2
 */
public interface CategoryService extends IService<Category> {
    /**
     * 查询分类列表
     * @param categories
     * @return
     */
    List<CategoryDTO> listCategories(List<Long> categories);

    List<Long> queryFirstLevelIds();

    /**
     * 查询分类
     * @param ids
     * @return
     */
    List<Category> getCategories(List<Long> ids);

    /**
     * 批量保存
     * @param categories
     */
    void saveBatchCategories(List<Category> categories);

    /**
     * 删除商家所有分类
     * @param merchant
     */
    void deleteByMerchantId(Long merchant);

    /**
     * 新增分类
     *
     * @param param 新增参数
     * @return 分类主键
     */
    Long addCategory(CategoryModifyParam param);

    /**
     * 修改分类
     *
     * @param id    分类主键
     * @param param 修改参数
     */
    void updateCategory(Long id, CategoryModifyParam param);

    /**
     * 查询分类
     *
     * @param param 查询参数
     * @return 分类分页信息
     */
    Pagination<CategoryDTO> getCategoryPaging(CategoryPagingParam param);

    /**
     * 获取有效分类信息
     *
     * @return 分类信息列表
     */
    List<CategoryDTO> listActiveCategories(CategoryListParam param);

    /**
     * 获取有效分类信息
     *
     * @return 分类信息列表
     */
    List<CategoryDTO> listCategory(CategoryListParam param);

    /**
     * 获取推荐分类信息
     *
     * @param size 分类个数
     * @return 分类信息列表
     */
    List<CategoryDTO> listRecommendCategories(int size);


    /**
     * 删除分类
     *
     * @param id    分类主键
     */
    void deleteCategory(Long id);

    /**
     * 获取分类列表
     * @return 分类信息
     */
    List<CategoryDTO> getRootCategories(CategoryListParam param);

    /**
     * 获取一级分类
     * @return 分类信息
     */
    List<CategoryDTO> getParentCategories(CategoryListParam param);

    /**
     * 获取商家目前已有的分类
     * @param merchantId 商家id
     * @return
     */
    List<CategoryDTO> getCategoriesByMerchant(Long merchantId);

    /**
     * 根据源id删除分类
     * @param ids
     */
    void deleteBySourceId(List<Long> ids);
}
