package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.dto.CurrencyRecordDTO;
import com.chelvc.cloud.vehicle.client.dto.EarningsDTO;
import com.chelvc.cloud.vehicle.server.copier.CurrencyRecordCopier;
import com.chelvc.cloud.vehicle.server.dao.CurrencyRecordMapper;
import com.chelvc.cloud.vehicle.server.entity.Asset;
import com.chelvc.cloud.vehicle.server.entity.CurrencyRecord;
import com.chelvc.cloud.vehicle.server.service.AssetService;
import com.chelvc.cloud.vehicle.server.service.CurrencyRecordService;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.DateUtils;
import com.chelvc.framework.common.util.StringUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 收益流水记录表操作接口
 *
 * @author liude
 * @date 2024/03/15
 */
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class CurrencyRecordServiceImpl extends ServiceImpl<CurrencyRecordMapper, CurrencyRecord> implements CurrencyRecordService {

    private final AssetService assetService;

    private final MerchantService merchantService;

     @Override
     public List<CurrencyRecordDTO> listCurrencyRecord(Integer type, Long date){
         Date localDate = new Date(date);
         SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
         Long userId = SessionContextHolder.getId();
         if (2 == type){
             List<Long> merchantIds = this.merchantService.getMerchantIdsByUserId(userId);
             if (CollectionUtils.isEmpty(merchantIds)){
                 return new ArrayList<>();
             }
             List<CurrencyRecord> currencyRecords = this.lambdaQuery()
                     .eq(CurrencyRecord::getUserId,merchantIds.get(0))
                     .eq(CurrencyRecord::getType, type)
                     .orderByDesc(CurrencyRecord::getCreateTime)
                     .list();//1808847197293170689
             List<CurrencyRecordDTO> list = CurrencyRecordCopier.INSTANCE.copying(currencyRecords);
             List<CurrencyRecordDTO> filteredPeople = list.stream()
                     .filter(item -> sdf.format(item.getCreateTime()).equals(sdf.format(localDate)))
                     .collect(Collectors.toList());
             return filteredPeople;
         }else{
             List<CurrencyRecord> currencyRecords = this.lambdaQuery()
                     .eq(CurrencyRecord::getUserId,userId)
                     .eq(CurrencyRecord::getType, type)
                     .orderByDesc(CurrencyRecord::getCreateTime)
                     .list();//1808847197293170689
             List<CurrencyRecordDTO> list = CurrencyRecordCopier.INSTANCE.copying(currencyRecords);
             List<CurrencyRecordDTO> filteredPeople = list.stream()
                     .filter(item -> !DateUtils.format(item.getCreateTime()).equals(DateUtils.format(localDate)))
                     .collect(Collectors.toList());
             return filteredPeople;
         }
     }
    @Override
    public EarningsDTO getCurrencyRecord(Integer type){
        Long userId = SessionContextHolder.getId();
        EarningsDTO currencyRecord = this.baseMapper.getCurrencyRecord(userId, type);
        if(currencyRecord == null){
            currencyRecord = new EarningsDTO();
        }
        Asset asset = assetService.queryAsset(userId, type);
        if(asset == null){
            currencyRecord.setWithdrawAmount(BigDecimal.ZERO);
        } else {
            currencyRecord.setWithdrawAmount(asset.getTotal());
        }
        return currencyRecord;
    }

    @Override
    public void recordFlow(Long userId, Integer type, Integer currencyType, Long orderId, Long sourceId,
                           Integer sourceType, BigDecimal amount, BigDecimal deduct, BigDecimal realityAmount) {
        CurrencyRecord currencyRecord = new CurrencyRecord();
        currencyRecord.setUserId(userId);
        currencyRecord.setType(type);
        currencyRecord.setCurrencyType(currencyType);
        currencyRecord.setOrderId(orderId);
        currencyRecord.setSourceId(sourceId);
        currencyRecord.setSourceType(sourceType);
        currencyRecord.setAmount(amount);
        currencyRecord.setDeduct(deduct);
        currencyRecord.setRealityAmount(realityAmount);
        currencyRecord.setCreateTime(LocalDateTime.now());
        baseMapper.insert(currencyRecord);
    }
}
