package com.chelvc.cloud.vehicle.server.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.constant.MerchantApplyStatus;
import com.chelvc.cloud.vehicle.client.param.SxyNotifyParam;
import com.chelvc.cloud.vehicle.server.dao.MerchantApplyRecordMapper;
import com.chelvc.cloud.vehicle.server.entity.MerchantApplyRecord;
import com.chelvc.cloud.vehicle.server.handle.TradeHandle;
import com.chelvc.cloud.vehicle.server.service.*;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.upay.sdk.Constants;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;

@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class MerchantApplyRecordServiceImpl extends
        ServiceImpl<MerchantApplyRecordMapper, MerchantApplyRecord> implements MerchantApplyRecordService {

    private final MerchantRelationService merchantRelationService;
    private final UserAuthService userAuthService;
    private final MerchantAuthService merchantAuthService;
    private final AssetService assetService;

    @Override
    public int insert(MerchantApplyRecord record) {
        return baseMapper.insert(record);
    }

    @Override
    public void updateSxyMerchantIdByNotify(SxyNotifyParam param) {
        JSONObject responseData = TradeHandle.decrypt(param);
        log.info("申请商户号回调参数：{}", responseData);
        String status = responseData.getString("subMerchantReviewStatus");
        if (Constants.ERROR.equals(status)) {
            //解密后报文，如果status=ERROR时，不需要验签
            return;
        }
        String requestId = responseData.getString("requestId");
        LambdaQueryWrapper<MerchantApplyRecord> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MerchantApplyRecord::getRequestId, requestId);
        MerchantApplyRecord record = baseMapper.selectOne(wrapper);
        Integer recordStatus = record.getStatus();
        if(recordStatus == 2){
            return;
        }
        String subMerchantId = responseData.getString("subMerchantId");
        Integer type = record.getType();
        Integer resultStatus = MerchantApplyStatus.getStatus(status);
        record.setStatus(resultStatus);
        record.setExamineStatus(status);
        record.setCertificateSupplementUrl(responseData.getString("certificateSupplementUrl"));
        record.setElectronicContractingUrl(responseData.getString("electronicContractingUrl"));
        record.setMerchantReviewRemarks(responseData.getString("merchantReviewRemarks"));
        record.setPostReviewRemark(responseData.getString("postReviewRemark"));
        record.setPostReviewStatus(responseData.getString("postReviewStatus"));
        record.setSubMerchantId(subMerchantId);
        record.setUpdateTime(new Date());
        DatabaseContextHolder.transactional((Executor) () -> {
            baseMapper.updateById(record);
            if(type == 1){
                //更改个人认证状态
                userAuthService.updateStatus(record.getUserId(), resultStatus);
            }
            if(type == 2){
                //更改商家认证状态
                merchantAuthService.updateStatus(record.getUserId(), resultStatus);
            }
            if("SUCCESS".equals(status)){
                int i = merchantRelationService.insert(record.getUserId(), record.getType(), subMerchantId);
                if(i == 1 && record.getType() == 1){
                    //用户申请商户号成功，平台转账给用户
                    BigDecimal total = assetService.queryTotal(record.getUserId(), 1);
                    assetService.transfer(0L, record.getUserId(), 1, total, "0");
                }
            }
        });
    }
}
