package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.user.client.UserClient;
import com.chelvc.cloud.user.client.model.Scope;
import com.chelvc.cloud.vehicle.client.dto.CategoryDTO;
import com.chelvc.cloud.vehicle.client.dto.MerchantAuthDTO;
import com.chelvc.cloud.vehicle.client.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.client.param.MerchantAuthModifyParam;
import com.chelvc.cloud.vehicle.client.param.MerchantAuthPagingParam;
import com.chelvc.cloud.vehicle.client.param.MerchantModifyParam;
import com.chelvc.cloud.vehicle.server.copier.MerchantAuthCopier;
import com.chelvc.cloud.vehicle.server.dao.MerchantAuthMapper;
import com.chelvc.cloud.vehicle.server.entity.MerchantAuth;
import com.chelvc.cloud.vehicle.server.service.CategoryService;
import com.chelvc.cloud.vehicle.server.service.MerchantAuthService;
import com.chelvc.cloud.vehicle.server.service.MerchantRelationService;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 商家认证业务操作实现
 *
 * @author liude
 * @date 2023/12/2
 */
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class MerchantAuthServiceImpl extends ServiceImpl<MerchantAuthMapper, MerchantAuth> implements MerchantAuthService{
    private final MerchantService merchantService;

    private final UserClient userClient;

    private final CategoryService categoryService;

    private final MerchantRelationService merchantRelationService;
    @Override
    public Long addMerchantAuth(@NonNull MerchantAuthModifyParam param) {
        Long userId = SessionContextHolder.getId();
        LambdaQueryWrapper<MerchantAuth> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(MerchantAuth::getApplicant, userId);
        if(baseMapper.selectCount(wrapper) > 0){
            throw new ResourceUnavailableException("审核认证已申请");
        }
        if (StringUtils.isEmpty(param.getStoreAddress())){
            param.setStoreAddress(param.getStoreAddress());
        }
        MerchantAuth merchantAuth = MerchantAuthCopier.INSTANCE.copying(param);
        MerchantAuth count = this.lambdaQuery().eq(MerchantAuth::getApplicant, userId).one();
        //AssertUtils.available(count <= 0, "商家认证已存在");
        if (null != count){
            this.updateMerchantAuth(count.getId(),param);
            return count.getId();
        }
        merchantAuth.setApplicationTime(new Date());
        merchantAuth.setApplicant(userId);
        merchantAuth.setReviewStatus("1");
        this.save(merchantAuth);
        MerchantModifyParam param1 = new MerchantModifyParam();
        param1.setName(param.getStoreName());
        param1.setLiaison(param.getLegalRepresentativeName());
        param1.setMobile(param.getMobileNumber());
        param1.setAddress(param.getStoreAddressDetail());
        param1.setRecommend(true);
        param1.setBusinessStatus("1");
        param1.setLogo(param.getStoreImage());
        param1.setOpening(param.getOpening());
        param1.setProvince(param.getProvince());
        param1.setCity(param.getCity());
        param1.setRegion(param.getRegion());
        param1.setDistrict(param.getDistrict());
        param1.setLatitude(param.getLatitude());
        param1.setLongitude(param.getLongitude());
        param1.setMainBusiness(param.getMainBusiness());
        param1.setAncillaryBusiness(param.getAncillaryBusiness());
        param1.setBanners(param.getStoreInnerImage());
        MerchantDTO merchant = this.merchantService.getMerchantByUserId(userId);
        if (ObjectUtils.isEmpty(merchant)){
            this.merchantService.addMerchant(param1);
        }else{
            this.merchantService.updateMerchant(merchant.getId(),param1);
        }
        //首信易商户子商户认证
        merchantRelationService.registerMerchantId(merchantAuth);
        return merchantAuth.getId();
    }

    @Override
    public void updateMerchantAuth(@NonNull Long id, @NonNull MerchantAuthModifyParam param) {
        if (StringUtils.isEmpty(param.getStoreAddress())){
            param.setStoreAddress(param.getStoreAddress());
        }
        MerchantAuth merchantAuth = AssertUtils.nonnull(this.getById(id), "商家认证不存在");
        String reviewStatus = merchantAuth.getReviewStatus();
        if("2".equals(reviewStatus)){
            throw new ResourceUnavailableException("已认证成功，不可重复认证");
        }
        MerchantAuthCopier.INSTANCE.copying(param, merchantAuth);
        merchantAuth.setReviewStatus("1");
        merchantAuth.setId(id);
        this.updateById(merchantAuth);
        MerchantModifyParam param1 = new MerchantModifyParam();
        param1.setName(param.getStoreName());
        param1.setLiaison(param.getLegalRepresentativeName());
        param1.setMobile(param.getMobileNumber());
        param1.setAddress(param.getStoreAddressDetail());
        param1.setRecommend(true);
        param1.setBusinessStatus("1");
        param1.setLogo(param.getStoreImage());
        param1.setOpening(param.getOpening());
        param1.setProvince(param.getProvince());
        param1.setCity(param.getCity());
        param1.setRegion(param.getRegion());
        param1.setDistrict(param.getDistrict());
        param1.setLatitude(param.getLatitude());
        param1.setLongitude(param.getLongitude());
        param1.setMainBusiness(param.getMainBusiness());
        param1.setAncillaryBusiness(param.getAncillaryBusiness());
        param1.setBanners(param.getStoreInnerImage());
        Long userId = SessionContextHolder.getId();
        List<Long> merchantIds = this.merchantService.getMerchantIdsByUserId(userId);
        if (CollectionUtils.isEmpty(merchantIds)){
            this.merchantService.addMerchant(param1);
        }else{
            this.merchantService.updateMerchant(merchantIds.get(0),param1);
        }
        //首信易商户子商户认证
        merchantRelationService.registerMerchantId(merchantAuth);
    }

    @Override
    public Pagination<MerchantAuthDTO> getMerchantAuthPaging(@NonNull MerchantAuthPagingParam param) {
        Long userId = null;
        Scope scope = com.chelvc.framework.common.util.StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
        }else {
            userId = SessionContextHolder.getId();
        }
        Page<MerchantAuth> page = this.lambdaQuery()
                .like(StringUtils.notEmpty(param.getStoreName()), MerchantAuth::getStoreName, param.getStoreName())
                .eq(StringUtils.notEmpty(userId),MerchantAuth::getApplicant,userId)
                .orderByDesc(MerchantAuth::getReviewStatus)
                .orderByAsc(MerchantAuth::getReviewTime).page(DatabaseContextHolder.page(param.getPaging()));
        return DatabaseContextHolder.pagination(page, MerchantAuthCopier.INSTANCE::copying);
    }

    @Override
    public MerchantAuthDTO queryById() {
        Long userId = SessionContextHolder.getId();
        MerchantAuth merchantAuth = this.lambdaQuery()
                .eq(MerchantAuth::getApplicant, userId).one();
        MerchantAuthDTO merchantAuthDTO = MerchantAuthCopier.INSTANCE.copying(merchantAuth);
        if (merchantAuthDTO == null){
            return null;
        }
        MerchantDTO merchantDTO = this.merchantService.getMerchantByUserId(userId);
        List<Long> categories = new ArrayList<>();
        categories.add(merchantDTO.getMainBusiness());
        List<CategoryDTO> list = this.categoryService.listCategories(categories);
        if (!CollectionUtils.isEmpty(list)){
            merchantDTO.setMainBusinessName(list.get(0).getName());
        }
        if (!CollectionUtils.isEmpty(merchantDTO.getAncillaryBusiness())){
            List<CategoryDTO> list2 = this.categoryService.listCategories(merchantDTO.getAncillaryBusiness());
            List<String> names = list2.stream().map(CategoryDTO::getName).collect(Collectors.toList());
            if (names.size() > 0){
                merchantDTO.setAncillaryBusinessName(String.join("/",names));
            }

        }
        merchantAuthDTO.setMerchant(merchantDTO);
        return merchantAuthDTO;
    }

    @Override
    public void certificationAudit(Long id, String state, String message) {
        Long userId = SessionContextHolder.getId();
        MerchantAuth merchantAuth = AssertUtils.nonnull(this.getById(id), "商家认证不存在");
        merchantAuth.setReviewStatus(state);
        merchantAuth.setReviewTime(new Date());
        merchantAuth.setReviewer(userId.toString());
        merchantAuth.setMessage(message);
        DatabaseContextHolder.transactional((Executor) () -> {
            this.updateById(merchantAuth);
            if (StringUtils.notEmpty(state) && "1".equals(state)) {
                //注册商家号
                merchantRelationService.registerMerchantId(merchantAuth);
            }
        });
    }

    @Override
    public int updateStatus(Long userId, Integer status) {
        UpdateWrapper<MerchantAuth> update = Wrappers.update();
        update.set("review_status", status);
        update.eq("applicant", userId);
        return baseMapper.update(null, update);
    }
}
