package com.chelvc.cloud.vehicle.server.service.impl;
import com.alibaba.fastjson.JSONObject;
import com.chelvc.cloud.user.client.UserClient;
import com.chelvc.cloud.user.client.dto.UserDTO;
import com.chelvc.cloud.vehicle.client.constant.TradeConstant;
import com.chelvc.cloud.vehicle.client.param.OmsOrderModifyParam;
import com.chelvc.cloud.vehicle.client.param.OrderReturnApplyModifyParam;
import com.chelvc.cloud.vehicle.server.entity.MerchantRelation;
import com.chelvc.cloud.vehicle.server.entity.OmsOrder;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderItem;
import com.chelvc.cloud.vehicle.server.service.AssetService;
import com.chelvc.cloud.vehicle.server.service.MerchantRelationService;
import com.chelvc.cloud.vehicle.server.service.OmsOrderRefundService;
import com.chelvc.cloud.vehicle.server.service.OnlinePayService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.wechat.PayChannel;
import com.chelvc.framework.wechat.PayMode;
import com.chelvc.framework.wechat.WechatUnifiedOrder;
import com.upay.sdk.entity.Payer;
import com.upay.sdk.entity.ProductDetail;
import com.upay.sdk.exception.HmacVerifyException;
import com.upay.sdk.exception.RequestException;
import com.upay.sdk.exception.ResponseException;
import com.upay.sdk.exception.UnknownException;
import com.upay.sdk.executer.ResultListenerAdpater;
import com.upay.sdk.onlinepay.builder.OrderBuilder;
import com.upay.sdk.onlinepay.builder.OrderResponse;
import com.upay.sdk.onlinepay.builder.QueryBuilder;
import com.upay.sdk.onlinepay.builder.RefundBuilder;
import com.upay.sdk.onlinepay.executer.OnlinePayOrderExecuter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.List;

/**
 * 首信易支付
 *
 * @author liude
 * @date 2025/1/6
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OnlinePayServiceImpl  implements OnlinePayService {
    private final UserClient userService;
    private final OmsOrderRefundService omsOrderRefundService;
    private final AssetService assetService;
    private final MerchantRelationService merchantRelationService;

    @Override
    public WechatUnifiedOrder order(OmsOrder omsOrder, List<OmsOrderItem> orderItemList, OmsOrderModifyParam orderParam){
        OrderBuilder builder = new OrderBuilder(TradeConstant.PARTNER_ID);
        Long orderAmount = omsOrder.getPayAmount().multiply(new BigDecimal("100")).longValue();
        String requestId = omsOrder.getId().toString();
        builder.setRequestId(omsOrder.getId().toString())
                .setOrderAmount(String.valueOf(orderAmount)) //单位：分
                .setOrderCurrency("CNY")
                .setNotifyUrl(TradeConstant.ORDER_NOTIFY_URL)
                .setCallbackUrl(String.format(TradeConstant.ORDER_CALLBACK_URL, requestId, TradeConstant.PARTNER_ID))
                .setRemark(omsOrder.getNote())
                .setPaymentModeCode(TradeConstant.MINIAPPS_WEIXIN_PAY_P2P)
                //.setAuthCode("")
                .setMerchantUserId(omsOrder.getUserId().toString())
                //.setBindCardId("")
                .setClientIp(SessionContextHolder.getHost());
        builder.setTimeout("15");
        //builder.setPaymentToken("");
        builder.setAppId(TradeConstant.APPID);

        //builder.setSerialNumber("");
        //builder.setApCardNo("");
        //builder.setApTransData("");
        //builder.setApUserData("");
        //builder.setReportSerialNo("");
        //builder.setReportTerminalNo("");
        //builder.setPartnerId(TradeConstant.PARTNER_ID);
        builder.setProjectId(String.valueOf(omsOrder.getMerchantId()));
        //builder.setSplitMark("");  //不传默认不分账


        //productDetails(商品信息)
//        List<ProductDetail> productDetails = new LinkedList();
        for (OmsOrderItem omsOrderItem : orderItemList) {
            ProductDetail productDetail = new ProductDetail();
            productDetail.setName(omsOrderItem.getGoodsName());
            productDetail.setAmount(omsOrderItem.getGoodsPrice().multiply(new BigDecimal("100")).longValue());
            productDetail.setQuantity(Long.valueOf(omsOrderItem.getGoodsQuantity()));
            builder.addProductDetail(productDetail);
        }
        UserDTO userDTO = userService.getUser(omsOrder.getUserId());
        builder.setOpenId(userDTO.getOpenid());
        //payer(身份信息)可选项
        Payer payer = new Payer();
        payer.setName(userDTO.getNickname());
        payer.setPhoneNum(userDTO.getMobile());
        payer.setBankCardNum("");
        payer.setIdNum("");
        payer.setIdType("");
        payer.setEmail("");
        builder.setPayer(payer);
        OrderResponse orderResponse = new OrderResponse();
        WechatUnifiedOrder wechatUnifiedOrder = new WechatUnifiedOrder();
        //分期支付参数
//        InstalmentParams instalmentParams = new InstalmentParams();
//        instalmentParams.setInterestType(interestType);
//        instalmentParams.setSuppBankCodes(suppBankCodes);
//        instalmentParams.setLimitNum(limitNum);
//        instalmentParams.setStoreNo(storeNo);
//        instalmentParams.setStoreName(storeName);
//        builder.setInstalmentParams(instalmentParams);
//        builder.setSubsidyMark(subsidyMark);
//        builder.setSubsidyAmount(subsidyAmount);
        String cause = "";
        try {
            OnlinePayOrderExecuter executer = new OnlinePayOrderExecuter();
            System.setProperty("https.protocols", "TLSv1.2,TLSv1.1,SSLv3");
            executer.bothOrder(builder, new ResultListenerAdpater() {
                /**
                 * 提交成功，不代表支付成功
                 */
                @Override
                public void success(JSONObject jsonObject) {
                    if (jsonObject.get("scanCode") != null) {
                        try {
                            orderResponse.setHmac(jsonObject.getString("hmac"));
                            orderResponse.setMerchantId(jsonObject.getString("merchantId"));
                            orderResponse.setRequestId(jsonObject.getString("requestId"));
                            orderResponse.setStatus(jsonObject.getString("status"));
                            String scanCodeStr = jsonObject.getString("scanCode");
//                            orderResponse.setScanCode(Base64.decodeBase64(scanCodeStr));
//                            session.setAttribute("orderResponse", orderResponse);
//                            resp.sendRedirect("scancode.jsp");
                        } catch (Exception e) {
                            log.error("响应异常："+e.getMessage());
                        }
                    } else if (jsonObject.get("jsString") != null) {
                        //微信小程序支付
                        try {
                            String jsString = jsonObject.getString("jsString");
                            JSONObject json = JSONObject.parseObject(jsString);
                            wechatUnifiedOrder.setAppid(json.getString("appId"));
                            wechatUnifiedOrder.setMchid(TradeConstant.MCH_ID);
                            wechatUnifiedOrder.setChannel(PayChannel.WECHAT);
                            wechatUnifiedOrder.setNonce(json.getString("nonceStr"));
                            wechatUnifiedOrder.setSign(json.getString("paySign"));
                            wechatUnifiedOrder.setTimestamp(json.getString("timeStamp"));
                            wechatUnifiedOrder.setAlgorithm(json.getString("signType"));
                            wechatUnifiedOrder.setPack(json.getString("package"));
                            wechatUnifiedOrder.setOrder(jsonObject.getString("requestId"));
                            wechatUnifiedOrder.setMode(PayMode.JSAPI);
                        } catch (Exception e) {
                            log.error("响应异常："+e.getMessage());
                        }
                    }
                }
                @Override
                public void redirect(JSONObject jsonObject, String redirectUrl) throws IOException {
//                    resp.sendRedirect(redirectUrl);
                }
            });
        } catch (ResponseException e) {
            JSONObject responseData = e.getResponseData();
            cause = responseData.getString("cause");
            log.error("transfer order error:", e);
        } catch (HmacVerifyException e) {
            cause = "签名验证异常";
            log.error("transfer order error:", e);
        } catch (RequestException e) {
            JSONObject responseData = e.getResponseData();
            cause = responseData.getString("cause");
            log.error("transfer order error:", e);
        } catch (UnknownException e) {
            cause = "未知异常";
            log.error("transfer order error:", e);
        }
        if(!StringUtils.isBlank(cause)){
            throw new ResourceUnavailableException(cause);
        }
        return wechatUnifiedOrder;
    }
    @Override
    public String query(OmsOrder omsOrder) {
        QueryBuilder builder = new QueryBuilder(omsOrder.getMerchantId().toString());
        builder.setRequestId(omsOrder.getId().toString());
        builder.setPartnerId(TradeConstant.PARTNER_ID);
        OnlinePayOrderExecuter executer = new OnlinePayOrderExecuter();
        try {
            executer.bothQuery(builder, new ResultListenerAdpater() {
                @Override
                public void success(JSONObject jsonObject) {
                    System.out.println("处理成功！<br/>");
                    String result = jsonObject.toJSONString();
                    System.out.print(result);
                }

                @Override
                public void failure(JSONObject jsonObject) {
                    System.out.println("处理失败！<br/>");
                    String result = jsonObject.toJSONString();
                    System.out.print(result);
                }

                @Override
                public void pending(JSONObject jsonObject) {
                    System.out.println("待处理！<br/>");
                    String result = jsonObject.toJSONString();
                    System.out.print(result);
                }
            });
        } catch (ResponseException e) {
            System.out.println("响应异常</br>");
            System.out.println(e.toString());
        } catch (HmacVerifyException e) {
            System.out.println("签名验证异常</br>");
            System.out.println(e.toString());
        } catch (RequestException e) {
            System.out.println("请求异常</br>");
            System.out.println(e.toString());
        } catch (UnknownException e) {
            System.out.println("未知异常</br>");
            System.out.println(e.toString());
        }
        return "";
    }
    @Override
    public void refund(OmsOrder omsOrder, Long refundId){
        //查询首信易商户号
        RefundBuilder builder = new RefundBuilder(TradeConstant.PARTNER_ID);
        builder.setRequestId(String.valueOf(refundId))
                .setAmount(String.valueOf(omsOrder.getPayAmount().multiply(new BigDecimal("100")).longValue()))
                .setOrderId(omsOrder.getOutOrderId())
                .setNotifyUrl(TradeConstant.ORDER_REFUND_URL);
        OnlinePayOrderExecuter executer = new OnlinePayOrderExecuter();
        String cause = "";
        try {
            executer.bothRefund(builder, new ResultListenerAdpater()  {
                @Override
                public void success(JSONObject jsonObject) {
                    //冻结平台金额
                    BigDecimal payAmount = omsOrder.getPayAmount();
                    //BigDecimal feeAmount = omsOrder.getFeeAmount();
                    //BigDecimal realAmount = payAmount.subtract(feeAmount);
                    int i = assetService.frozenAmount(payAmount, 0L, 0);
                    if(i == 1){
                        omsOrderRefundService.updateStatus(refundId, 1, "");
                        return;
                    }
                    omsOrderRefundService.updateStatus(refundId, 3, "冻结金额失败");
                    throw new ResourceUnavailableException("退款申请失败");
                }
            });
            return;
        } catch (ResponseException e) {
            JSONObject responseData = e.getResponseData();
            cause = responseData.getString("cause");
            log.error("transfer order error:", e);
        } catch (HmacVerifyException e) {
            cause = "签名验证异常";
            log.error("transfer order error:", e);
        } catch (RequestException e) {
            JSONObject responseData = e.getResponseData();
            cause = responseData.getString("cause");
            log.error("transfer order error:", e);
        } catch (UnknownException e) {
            cause = "未知异常";
            log.error("transfer order error:", e);
        }
        omsOrderRefundService.updateStatus(refundId, 3, cause);
        throw new ResourceUnavailableException(cause);
    }
    @Override
    public String refundQuery(OmsOrder omsOrder){
        RefundBuilder builder = new RefundBuilder(omsOrder.getMerchantId().toString());
        builder.setRequestId(omsOrder.getId().toString());
        OnlinePayOrderExecuter executer = new OnlinePayOrderExecuter();
        try {
            executer.bothRefundQuery(builder, new ResultListenerAdpater()   {
                @Override
                public void success(JSONObject jsonObject) {
                    System.out.println("处理成功！<br/>");
                    String result = jsonObject.toJSONString();
                    System.out.print(result);
                }

                @Override
                public void failure(JSONObject jsonObject) {
                    System.out.println("处理失败！<br/>");
                    String result = jsonObject.toJSONString();
                    System.out.print(result);
                }

                @Override
                public void pending(JSONObject jsonObject) {
                    System.out.println("待处理！<br/>");
                    String result = jsonObject.toJSONString();
                    System.out.print(result);
                }
            });
        } catch (ResponseException e) {
            System.out.println("响应异常</br>");
            System.out.println(e.toString());
        } catch (HmacVerifyException e) {
            System.out.println("签名验证异常</br>");
            System.out.println(e.toString());
        } catch (RequestException e) {
            System.out.println("请求异常</br>");
            System.out.println(e.toString());
        } catch (UnknownException e) {
            System.out.println("未知异常</br>");
            System.out.println(e.toString());
        }
        return "";
    }
    @Override
    public String cancelOrder(OmsOrder omsOrder){
        QueryBuilder builder = new QueryBuilder(omsOrder.getMerchantId().toString());
        builder.setRequestId(omsOrder.getId().toString());
        builder.setPartnerId(TradeConstant.PARTNER_ID);
        OnlinePayOrderExecuter executer = new OnlinePayOrderExecuter();
        try {
            executer.bothCancelOrder(builder, new ResultListenerAdpater()   {
                @Override
                public void success(JSONObject jsonObject) {
                    System.out.println("处理成功！<br/>");
                    String result = jsonObject.toJSONString();
                    System.out.print(result);
                }

                @Override
                public void failure(JSONObject jsonObject) {
                    System.out.println("处理失败！<br/>");
                    String result = jsonObject.toJSONString();
                    System.out.print(result);
                }

                @Override
                public void pending(JSONObject jsonObject) {
                    System.out.println("待处理！<br/>");
                    String result = jsonObject.toJSONString();
                    System.out.print(result);
                }
            });
        } catch (ResponseException e) {
            System.out.println("响应异常</br>");
            System.out.println(e.toString());
        } catch (HmacVerifyException e) {
            System.out.println("签名验证异常</br>");
            System.out.println(e.toString());
        } catch (RequestException e) {
            System.out.println("请求异常</br>");
            System.out.println(e.toString());
        } catch (UnknownException e) {
            System.out.println("未知异常</br>");
            System.out.println(e.toString());
        }
        return "";
    }
}
