package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.dto.UserAuthDTO;
import com.chelvc.cloud.vehicle.client.param.UserAuthParam;
import com.chelvc.cloud.vehicle.server.dao.UserAuthMapper;
import com.chelvc.cloud.vehicle.server.entity.UserAuth;
import com.chelvc.cloud.vehicle.server.service.MerchantRelationService;
import com.chelvc.cloud.vehicle.server.service.UserAuthService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UserAuthServiceImpl extends ServiceImpl<UserAuthMapper, UserAuth> implements UserAuthService {
    private final MerchantRelationService merchantRelationService;
    @Override
    public UserAuthDTO getUserAuthInfo(Long userId) {
        if(userId == null){
            throw new ResourceUnavailableException("请重新登录账号");
        }
        UserAuthDTO userAuthInfo = baseMapper.getUserAuthInfo(userId);
        if(userAuthInfo == null){
            return UserAuthDTO.init(0);
        }
        return userAuthInfo;
    }

    @Override
    public void userAuth(UserAuthParam param) {
        Long userId = SessionContextHolder.getId();
        LambdaQueryWrapper<UserAuth> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(UserAuth::getUserId, userId);
        UserAuth userAuth = baseMapper.selectOne(wrapper);
        if(userAuth == null){
            userAuth = new UserAuth();
            BeanUtils.copyProperties(param, userAuth);
            userAuth.setUserId(userId);
            userAuth.setCreateTime(LocalDateTime.now());
            userAuth.setStatus(1);
            baseMapper.insert(userAuth);
        } else {
            Integer status = userAuth.getStatus();
            if(status == 2){
                throw new ResourceUnavailableException("已认证成功，不可重复认证");
            }
            userAuth.setBankCardNo(param.getBankCardNo());
            userAuth.setIdCardBackPhoto(param.getIdCardBackPhoto());
            userAuth.setIdCardFrontPhoto(param.getIdCardFrontPhoto());
            userAuth.setUserName(param.getUserName());
            userAuth.setProvince(510000);   //四川省
            userAuth.setCity(510100);       //成都市
            userAuth.setDistrict(510116);   //双流区
            userAuth.setPhone(param.getPhone());
            userAuth.setEmail("chelvc@qq.com");
            userAuth.setAddress("四川省成都市双流区");
            userAuth.setIdCardExpirationDateStart(param.getIdCardExpirationDateStart());
            userAuth.setIdCardExpirationDate(param.getIdCardExpirationDate());
            userAuth.setAccountName(param.getUserName());
            userAuth.setIdCardNo(param.getIdCardNo());
            userAuth.setStatus(1);
            baseMapper.updateById(userAuth);
        }
        //首信易个人子商户认证
        merchantRelationService.registerMerchantId(userAuth);
    }

    @Override
    public int updateStatus(Long userId, Integer status) {
        UpdateWrapper<UserAuth> update = Wrappers.update();
        update.set("status", status);
        update.eq("user_id", userId);
        return baseMapper.update(null, update);
    }
}
